/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of rrd_agent.
	
    rrd_agent is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    rrd_agent is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <string>
#ifndef _WIN32
#include <syslog.h>
#endif
#include "tinyxml.h"
#include <log4cplus/logger.h>



bool load_config(const std::string & filename, std::string & rrd_path, std::string & rra_defs,  std::string & endpoint, std::string & msg_format)
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	TiXmlDocument doc(filename.c_str());
	doc.LoadFile();
	std::ostringstream log_message;
	log_message << "invalid config file "<< filename << "(" << doc.ErrorDesc() << ")";
	if(doc.Error()) {	
		LOG4CPLUS_ERROR(logger, log_message);
		return false;	
	}
	TiXmlNode *root = doc.FirstChild("config");
	if(!root) {
		LOG4CPLUS_ERROR(logger, "invalid config file (no root element)");
		return false;	
	}
	TiXmlNode *rrd_path_node = root->FirstChild("rrd_path");
	if(rrd_path_node) {
		rrd_path = rrd_path_node->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger, "rrd path not specified in config");
		return false;
	}
	
	TiXmlNode *msg_format_node = root->FirstChild("msg_format");
	if(msg_format_node) {
		msg_format = msg_format_node->FirstChild()->Value();
	}
	
	TiXmlNode *ep = root->FirstChild("endpoint");
	if(ep) {
		endpoint = ep->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger, "endpoint not specified in config");
		return false;
	}
	return true;
}

