/************************************************************************
	Copyright (C) 2010 Robert Scott.
	
	This file if part of rrd_agent.
	
    rrd_agent is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    rrd_agent is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
*************************************************************************/

#include "config.h"
#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/xpressive/xpressive.hpp>
#include <zmq.hpp>

#include <log4cplus/logger.h>
#include <log4cplus/loglevel.h>
#include <log4cplus/configurator.h>
#include <log4cplus/layout.h>

#include <json/value.h>
#include <json/reader.h>


#ifndef _WIN32
#include <signal.h>
#include <string.h>
#include <unistd.h>
extern sig_atomic_t run;
#else
extern int run;
#endif

std::string  parse(const std::string & data);
bool rrd_process(const std::string & data_folder, const std::string & rra_defs, const std::string & time, const std::string & logger_code, const std::string & monitor_code, int channel_number, const std::string & channel_type, const std::string & channel_value);	
void update_from_json(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root);
void update_from_json_multi(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root);
void update_from_json_single(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root);

void process_messages(const std::string & endpoint, const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format) 
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	zmq::context_t context(1);
	zmq::socket_t socket(context, ZMQ_SUB);
	socket.setsockopt(ZMQ_SUBSCRIBE,"",0);
	socket.connect(endpoint.c_str());
	std::ostringstream log_message;
	while(run) {
		zmq::message_t message;
		
		int socket_rslt;
		try {
			socket_rslt = socket.recv(&message);
		}
		catch(std::exception & e) {
			LOG4CPLUS_DEBUG(logger, e.what());
			break;
		}
		if(socket_rslt) {
			std::string payload= (char *)message.data();
			Json::Reader reader;
			std::string json = parse(payload);
			if(json.length()>0) {
				Json::Value root;
				if(!reader.parse(json, root)) {
					log_message.str("");
					log_message << "json parse failed (" << json << ")";
					LOG4CPLUS_DEBUG(logger, log_message.str());
					continue;
				}	
				update_from_json(rrd_path, rra_def_file, msg_format, root);
	
			}
		}
	}
}

void update_from_json(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root)
{
	if(msg_format == "json-multi") {
		update_from_json_multi(rrd_path, rra_def_file, msg_format, root);
	}
	if(msg_format == "json-single") {
		update_from_json_single(rrd_path, rra_def_file, msg_format, root);
	}
}

void update_from_json_multi(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root)
{
	std::string logger_code = root["logger"].asString();
	std::string time = root["time"].asString();		
	Json::Value monitors = root["monitor"];
	for(int index1 = 0; index1!= monitors.size(); ++index1) {
		std::string monitor_code = monitors[index1]["code"].asString();
		int logger_channel_number = monitors[index1]["channel"].asInt();	
		Json::Value sensors = monitors[index1]["sensor"];
		for(int index2=0; index2!=sensors.size(); ++index2) {
			int channel_number = sensors[index2]["channel"].asInt();
			std::string channel_type = sensors[index2]["type"].asString();
			std::string channel_value = sensors[index2]["value"].asString();
			if(channel_value!="U") {
				rrd_process(rrd_path, rra_def_file, time, logger_code, monitor_code, channel_number, channel_type, channel_value);	
			}
		}
	}	
}

void update_from_json_single(const std::string & rrd_path, const std::string & rra_def_file, const std::string & msg_format, const Json::Value & root)
{
	std::string logger_code = root["logger"].asString();
	std::string time = root["time"].asString();	
	std::string monitor_code = root["monitor"]["code"].asString();
	int logger_channel_number = root["monitor"]["channel"].asInt();	
	int sensor_channel_number = root["sensor"]["channel"].asInt();
	std::string channel_type = root["sensor"]["type"].asString();
	std::string channel_value = root["sensor"]["value"].asString();
	rrd_process(rrd_path, rra_def_file, time, logger_code, monitor_code, sensor_channel_number, channel_type, channel_value);	
}

std::string  parse(const std::string & data)
{
	std::string json;
	boost::xpressive::sregex rex = boost::xpressive::sregex::compile( "^[a-zA-Z0-9]*({.*)" );
	boost::xpressive::smatch what;
	if( boost::xpressive::regex_match( data, what, rex ) ) {
		json =  what[1].str();
	}
	return json;
}